SystemOrganization addCategory: #FuelBenchmarks!
SystemOrganization addCategory: #'FuelBenchmarks-Samples'!
SystemOrganization addCategory: #'FuelBenchmarks-Tests'!

FLSerializationTest subclass: #FLBenchmarkSampleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarkSampleTest methodsFor: 'running' stamp: 'MartinDias 1/1/2012 20:19'!
assertSample: aSymbol  usesIndexesOf: numberOfBytes

	| analysis |
	analysis := FLAnalyzer newDefault analysisFor: (FLBenchmarkSample fromSelector: aSymbol).
	
	self assert: analysis clusterization objectCount digitLength = numberOfBytes.! !

!FLBenchmarkSampleTest methodsFor: 'tests' stamp: 'MartinDias 8/8/2011 15:20'!
testAllSamples
	"Tests that the result of serialization and materialization of every sample does not raise an error and that it has the expected size. Would be nice to check if the result is equal but many classes do not have = well defined for us."

	FLBenchmarkSample allSamples do: [ :aSample | self testSample: aSample ]! !

!FLBenchmarkSampleTest methodsFor: 'tests' stamp: 'MartinDias 1/1/2012 20:33'!
testIndexesSamples

	self assertSample: #indexedWithOneByte usesIndexesOf: 1.
	self assertSample: #indexedWithTwoBytes usesIndexesOf: 2.
	self assertSample: #indexedWithThreeBytes usesIndexesOf: 3.! !

!FLBenchmarkSampleTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:18'!
testSample: aSample
	"Tests that the result of serialization and materialization has the expected size. Would be nice to check if the result is equal but many classes do not have = well defined for us."

	| result |
	self 
		shouldnt: [ result := self resultOfSerializeAndMaterialize: aSample content ] 
		raise: Error.
	
	self assert: aSample content size = result size.! !

FLSerializationTest subclass: #FLBenchmarksTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarksTest classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/31/2011 12:03'!
packageNamesUnderTest
	^ #('FuelBenchmarks')! !

!FLBenchmarksTest methodsFor: 'running' stamp: 'MartinDias 6/2/2011 19:34'!
benchmarksWithTwoRunnersAndTwoSamples
	
	| benchmarks |
	benchmarks := FLBenchmarks new.
	benchmarks runners add: (FLBenchmarksDummyRunner titled: 'A').
	benchmarks runners add: (FLBenchmarksDummyRunner titled: 'B').
	benchmarks samples add: (FLBenchmarkSample title: 'sample1' content: #(1 2 3 4)).
	benchmarks samples add: (FLBenchmarkSample title: 'sample2' content: #(5 6 7 8)).
	^ benchmarks! !

!FLBenchmarksTest methodsFor: 'running' stamp: 'MartinDias 10/5/2011 20:27'!
fuelBenchmarksWithSmallSample
	^ FLBenchmarks new
		samples: (FLBenchmarkSample fromSelectors: #(#classSamples));
		runners: (Array with: FLFuelBenchmarkRunner new);
		yourself! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 18:19'!
testNumberOfResults
	"Simulating the real runners, each dummy adds two results: the serialization one and the materialization one."
	
	| benchmarks |
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	self assert: 8 = benchmarks results size! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 6/2/2011 20:03'!
testPrintCSV
	
	| benchmarks aStream |
	aStream := '' writeStream.
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	benchmarks printResultsOnCSV: aStream.
	aStream close.

	self deny: aStream contents isEmpty! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 6/2/2011 19:52'!
testPrintClassic
	
	| benchmarks aStream |
	aStream := '' writeStream.
	benchmarks := self benchmarksWithTwoRunnersAndTwoSamples.
	benchmarks run.
	benchmarks printResultsOn: aStream.
	aStream close.

	self deny: aStream contents isEmpty! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 20:50'!
testUseFileStream

	| benchmarks |
	benchmarks := self fuelBenchmarksWithSmallSample.
	self shouldnt: [ 
			benchmarks
				useFileStream;
				run ]
		raise: Error.
	benchmarks results do: [:aResult | 
		self assert: aResult successful].! !

!FLBenchmarksTest methodsFor: 'tests' stamp: 'MartinDias 10/5/2011 21:02'!
testUseMemoryStream

	| benchmarks |
	benchmarks := self fuelBenchmarksWithSmallSample.
	self shouldnt: [ 
			benchmarks
				useMemoryStream;
				run ]
		raise: Error.
	benchmarks results do: [:aResult | 
		self assert: aResult successful].! !

!Time class methodsFor: '*FuelBenchmarks' stamp: 'MarianoMartinezPeck 6/30/2011 10:32'!
millisecondsToRunWithoutContextSwitch: timedBlock 
	"Answer the number of milliseconds timedBlock takes to return its value."

	| initialMilliseconds |
	initialMilliseconds := self millisecondClockValue.
	timedBlock valueNoContextSwitch.
	^self millisecondsSince: initialMilliseconds! !

Object subclass: #FLBenchmarkResult
	instanceVariableNames: 'run milliseconds title successful typeOfOperation serializationSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkResult commentStamp: 'MartinDias 10/24/2011 12:15' prior: 0!
I am a result produced by FLBenchmark when running certain sample with certain runner.!

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 20:27'!
isMaterialization
	^ typeOfOperation = #materialization! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 11:04'!
isSerialization
	^ typeOfOperation = #serialization! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
milliseconds

	^ milliseconds! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
milliseconds: anObject

	milliseconds := anObject! !

!FLBenchmarkResult methodsFor: 'printing' stamp: 'MarianoMartinezPeck 9/28/2011 17:47'!
printOn: aStream

	aStream
		nextPutAll: title;
		nextPutAll: ' sample: ';
		nextPutAll: run sample title;
		nextPutAll: ' size: ';
		nextPutAll: run sample content size printString.
	
	successful 
		ifTrue: [
			aStream 
				nextPutAll: ' ms: ';
				nextPutAll: milliseconds printString.
			self isMaterialization ifTrue: [
				aStream 
					nextPut: Character cr;
					nextPutAll: 'Serialization size: ';
					nextPutAll: serializationSize printString ]] 
		ifFalse: [
			aStream nextPutAll: ' FAILED' ]
! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:30'!
run
	^ run! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:30'!
run: anObject
	run := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 11:02'!
sampleSize

	^ run sample content size! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 6/17/2011 22:37'!
serializationSize
	^ serializationSize! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 6/17/2011 22:38'!
serializationSize: anObject
	serializationSize := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MD 12/3/2010 17:06'!
successful

	^ successful! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MD 12/3/2010 17:07'!
successful: aBoolean

	successful := aBoolean! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
title

	^ title! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MartinDias 11/3/2010 23:07'!
title: anObject

	title := anObject! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 11:03'!
typeOfOperation

	^ typeOfOperation ! !

!FLBenchmarkResult methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 11:02'!
typeOfOperation: anObject

	typeOfOperation := anObject! !

Object subclass: #FLBenchmarkRun
	instanceVariableNames: 'sample runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkRun commentStamp: 'MartinDias 10/24/2011 12:13' prior: 0!
I represent a benchmark run of certain sample with certain runner.!

!FLBenchmarkRun classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/6/2010 17:48'!
sample: aSample runner: aRunner

	^ self basicNew
		initialize;
		sample: aSample;
		runner: aRunner;
		yourself		! !

!FLBenchmarkRun methodsFor: 'running' stamp: 'mariano 5/31/2011 11:57'!
runOn: benchmarks

	runner runOn: benchmarks with: self! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
runner
	^ runner! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
runner: anObject
	runner := anObject! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
sample
	^ sample! !

!FLBenchmarkRun methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 17:49'!
sample: anObject
	sample := anObject! !

Object subclass: #FLBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkRunner commentStamp: 'MarianoMartinezPeck 6/17/2011 14:10' prior: 0!
All FLBenchmarkRunner can be tested like this:

| aStream |
aStream :=  (MultiByteBinaryOrTextStream on: '').
FLXXXBenchmarkRunner new serialize: 'mariano' on: aStream.
aStream reset.
FLXXXBenchmarkRunner new materializeFrom: aStream.!

!FLBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:40'!
allRunners

	^ self allSubclasses 
		select: [ :aRunnerClass | aRunnerClass shouldRun ] 
		thenCollect: [ :aRunnerClass | aRunnerClass new ]! !

!FLBenchmarkRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:39'!
shouldRun

	^ true! !

!FLBenchmarkRunner methodsFor: 'testing' stamp: 'MarianoMartinezPeck 5/17/2011 13:20'!
isFuelRunner 
	^ false! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 5/29/2011 19:12'!
materializeFrom: aStream
	"Hook method for do materializing. The time for that will be included in the benchmarks results."

	self subclassResponsibility! !

!FLBenchmarkRunner methodsFor: 'running' stamp: 'MartinDias 10/6/2011 00:10'!
runOn: benchmarks with: aRun

	| serializationSize |
	benchmarks addResultRunningSerialization: [
		| aStream |
		aStream := (benchmarks writeStreamNamed: self runnerName).
		[ self serialize: aRun sample content on: aStream ]
			ensure: [
				serializationSize := aStream reset size.
				aStream close ] ]
	titled: self runnerName, ' Serialization' run: aRun.
	
	benchmarks addResultRunningMaterialization: [
		| aStream |
		aStream := (benchmarks readStreamNamed: self runnerName).
		[ self materializeFrom: aStream ]
			ensure: [ aStream close ] ] 
	titled: self runnerName, ' Materialization' run: aRun serializationSize: serializationSize.! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 16:08'!
runnerName
	"Hook method for the name"

	self subclassResponsibility! !

!FLBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 16:07'!
serialize: anObject on: aStream
	"Hook method for do serializing. The time for that will be included in the benchmarks results."

	self subclassResponsibility! !

FLBenchmarkRunner subclass: #FLBenchmarksDummyRunner
	instanceVariableNames: 'title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Tests'!

!FLBenchmarksDummyRunner commentStamp: 'MartinDias 10/24/2011 12:13' prior: 0!
I am a runner for tests.!

!FLBenchmarksDummyRunner classSide methodsFor: 'accessing' stamp: 'MartinDias 12/7/2010 10:41'!
shouldRun 
	"Returns false so it's not added to the benchmarks default run'"

	^ false! !

!FLBenchmarksDummyRunner classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/2/2011 19:34'!
titled: aString

	^ super new
		title: aString;
		yourself! !

!FLBenchmarksDummyRunner methodsFor: 'running' stamp: 'MartinDias 6/17/2011 22:46'!
runOn: benchmarks with: aRun

	benchmarks addResultRunningSerialization: [ ] titled: self title, ' Serialization' run: aRun.
	benchmarks addResultRunningMaterialization: [ ] titled: self title, ' Materialization' run: aRun serializationSize: 10.
! !

!FLBenchmarksDummyRunner methodsFor: 'hooks' stamp: 'MartinDias 6/2/2011 19:53'!
runnerName

	^ title! !

!FLBenchmarksDummyRunner methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:34'!
title
	^ title! !

!FLBenchmarksDummyRunner methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:34'!
title: anObject
	title := anObject! !

FLBenchmarkRunner subclass: #FLFuelBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLFuelBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using Fuel.

See FLBenchmarks for instructions for run.!

!FLFuelBenchmarkRunner methodsFor: 'testing' stamp: 'MarianoMartinezPeck 5/17/2011 13:20'!
isFuelRunner 
	^ true! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 9/11/2011 21:06'!
materializeFrom: aStream

	^(FLMaterializer newDefault materializeFrom: aStream binary) root! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:36'!
runnerName

	^ 'Fuel'! !

!FLFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 9/13/2011 23:13'!
serialize: anObject on: aStream

	FLSerializer newDefault serialize: anObject on: aStream binary! !

FLBenchmarkRunner subclass: #FLFuelGZipBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLFuelGZipBenchmarkRunner commentStamp: 'MartinDias 12/9/2011 15:52' prior: 0!
I serialize and materialize objects for benchmarks using Fuel configured for using gzip compression.

See FLBenchmarks>>benchFuelGZipped for an example.!

!FLFuelGZipBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 12/9/2011 16:21'!
materializeFrom: aStream

	^(FLMaterializer newDefault readGZip materializeFrom: aStream binary) root! !

!FLFuelGZipBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 12/9/2011 14:58'!
runnerName

	^ 'FuelGZipped'! !

!FLFuelGZipBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 12/9/2011 16:13'!
serialize: anObject on: aStream

	FLSerializer newDefault writeGZip serialize: anObject on: aStream binary! !

FLBenchmarkRunner subclass: #FLFullFuelBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLFullFuelBenchmarkRunner commentStamp: 'MartinDias 9/25/2011 22:00' prior: 0!
I serialize and materialize objects for benchmarks using Fuel considering globals as much as possible.

See FLBenchmarks for instructions for run.!

!FLFullFuelBenchmarkRunner methodsFor: 'testing' stamp: 'MartinDias 9/25/2011 19:57'!
isFuelRunner 
	^ true! !

!FLFullFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 9/25/2011 19:57'!
materializeFrom: aStream

	^(FLMaterializer newDefault materializeFrom: aStream binary) root! !

!FLFullFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 9/25/2011 19:58'!
runnerName

	^ 'Full Fuel'! !

!FLFullFuelBenchmarkRunner methodsFor: 'hooks' stamp: 'MartinDias 9/25/2011 22:03'!
serialize: anObject on: aStream

	FLSerializer newFull serialize: anObject on: aStream binary! !

FLBenchmarkRunner subclass: #FLImageSegmentBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLImageSegmentBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using ImageSegment.

See FLBenchmarks for instructions for run.!

!FLImageSegmentBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 5/30/2011 16:35'!
materializeFrom: aStream
	aStream binary.
	^ aStream fileInObjectAndCode arrayOfRoots first! !

!FLImageSegmentBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 5/30/2011 15:22'!
runnerName
	^ 'ImageSegment'! !

!FLImageSegmentBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 6/10/2011 10:49'!
serialize: anObject on: aStream
	| segment |
	aStream binary.
	segment := ImageSegment new copyFromRootsForExport: (Array with: anObject).
	"Transcript show: 'Total outPointers: ', segment outPointers size asString; cr."
	"Transcript show: 'Total content: ', anObject size asString; cr."
	segment writeForExportOn: aStream.

	

	! !

FLBenchmarkRunner subclass: #FLReferenceStreamBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLReferenceStreamBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using ReferenceStream.

See FLBenchmarks for instructions for run.!

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 6/4/2011 14:40'!
materializeFrom: aStream
	aStream binary.
	^ (ReferenceStream on: aStream) next! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:37'!
runnerName

	^ 'ReferenceStream'! !

!FLReferenceStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 5/29/2011 19:10'!
serialize: anObject on: aStream
	aStream binary.
	(ReferenceStream on: aStream) nextPut: anObject ! !

FLBenchmarkRunner subclass: #FLSmartRefStreamBenchmarkRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLSmartRefStreamBenchmarkRunner commentStamp: 'MartinDias 6/2/2011 00:59' prior: 0!
I serialize and materialize objects for benchmarks using SmartRefStream.

See FLBenchmarks for instructions for run.!

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 6/4/2011 14:40'!
materializeFrom: aStream
	aStream binary.
	^ aStream fileInObjectAndCode! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MD 12/3/2010 15:38'!
runnerName

	^ 'SmartRefStream'! !

!FLSmartRefStreamBenchmarkRunner methodsFor: 'hooks' stamp: 'MarianoMartinezPeck 5/29/2011 19:10'!
serialize: anObject on: aStream
	aStream binary.
	(SmartRefStream on: aStream) nextPut: anObject! !

Object subclass: #FLBenchmarkSample
	instanceVariableNames: 'content title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkSample commentStamp: 'MartinDias 6/1/2011 19:38' prior: 0!
I represent a benchmark sample.

The common way of create instances of me is:

FLBenchmarkSample fromSelector: #completeBinaryTree.

Also you can create a bunch of instances with:

FLBenchmarkSample basicSamples.!

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 09:55'!
advancedSampleSelectors
	"Symbols are put here on purpose. Some serializers like ImageSegment can have problems with them"
	^ self class methodsInCategory: 'private advanced samples'! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'mariano 5/31/2011 14:51'!
allSampleSelectors
		
	^ self advancedSampleSelectors, self basicSampleSelectors ! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/15/2011 15:31'!
allSamples

	^self fromSelectors: self allSampleSelectors! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 16:37'!
allSmallSamples

	 | allNonCollectionSamples |
	allNonCollectionSamples := OrderedCollection new.
			self smallSamplesDo:
				[ : eachSelector : eachSamples |
				allNonCollectionSamples addAll: eachSamples ] .
	^ allNonCollectionSamples! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive collection samples' stamp: 'MarianoMartinezPeck 10/18/2011 15:41'!
arraySamples

	| all |
	all := OrderedCollection new.
	30 timesRepeat: [all add: self allSmallSamples asArray].
	^ all asArray
	
	
! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive collection samples' stamp: 'MarianoMartinezPeck 9/6/2011 23:17'!
associationsSamples
	^ self populatedDictionary associations
! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
atomicSamples

	^Array
		with: nil
		with: true
		with: false
! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive collection samples' stamp: 'MarianoMartinezPeck 10/18/2011 16:07'!
bagSamples
	|  all |
	all := OrderedCollection new.
	30 timesRepeat: [ 	all add: (Bag new addAll: (self allSmallSamples); yourself) ].
	^ all asArray! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 16:31'!
basicSampleSelectors
			
	^ self primitiveSampleSelectors, self nonPrimitiveSampleSelectors! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/15/2011 15:30'!
basicSamples

	^self fromSelectors: self basicSampleSelectors! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 15:01'!
bitmaps
	| numbers bitmaps |
	numbers := OrderedCollection new.
	bitmaps := OrderedCollection new.
	numbers addAll:  ( 1 << 30 to:( ( 1 << 30) + 10000) ) asArray.
	numbers do: [:each | bitmaps add: (Bitmap with: each with: each + 1) ].
	^ bitmaps
		

! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 9/28/2011 17:22'!
blockSamples
	^ {[ : a : b | a name > b name ]}! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
byteArraySamples

	^Array
		with: ByteArray new
		with: (ByteArray new: 100)
		with: (0 to: 255) asByteArray
! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:20'!
byteArrays
	
	^ (1 to: 1 << 15) 
		collect: [ :index | (index * 123456790) asByteArray  ]! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
characterSamples

	^Character allCharacters! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/3/2011 15:53'!
characters
	| all |
	all := OrderedCollection new.
	(1 << 16) timesRepeat: [
	all add: (1 to: 255 ) atRandom asCharacter ].
	^ all ! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 9/28/2011 17:48'!
classSamples
	
	^ Collection withAllSubclasses asArray! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:41'!
completeBinaryTree 
	| aCollection numberOfLeafs |
	numberOfLeafs := 2 raisedTo: 13.
	aCollection := OrderedCollection new.
	1 to: numberOfLeafs do: [ :i | 
		aCollection add: (
			FLPair new 
				left: (2 * i); 
				right: (2 * i) + 1;
				yourself) ].
	
	[aCollection size > 1] 
		whileTrue: [
			aCollection := aCollection pairsCollect: [ :leftPair :rightPair | 
				FLPair new 
					left: leftPair;
					right: rightPair;
					yourself ] ].

	^ aCollection! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 9/1/2011 21:26'!
dateAndTimes
	| times initialTime initialDate |
	times := OrderedCollection new.
	initialTime := Time fromSeconds: 76020.
	initialDate := Date fromSeconds: 3492288000.
	1 to: 20000 do: [:index |
		times add: (DateAndTime date: (initialDate subtractDays: index) time: (initialTime addSeconds: index)).
		].
	^ times! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 9/1/2011 21:06'!
dates
	| dates initialDate |
	dates := OrderedCollection new.
	initialDate := Date fromSeconds: 3492288000.
	1 to: 10000 do: [:index |
		dates add: (initialDate subtractDays: index).
		].
	^ dates! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive collection samples' stamp: 'MarianoMartinezPeck 9/6/2011 23:16'!
dictionarySamples
	
	^Array
		with: Dictionary new
		with: self populatedDictionary
		with: Smalltalk! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 9/1/2011 23:18'!
durations
	^ self times collect: [:each | each asDuration]! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
exceptionSamples
	^ { Error new messageText: 'sample error'.
	Warning new messageText: 'sample warning'.
	Notification new messageText: 'sample notification' }! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
floatSamples

	^{ Float e.  Float halfPi.  Float infinity.  "Float nan. <- this one is not supported, see http://lists.squeakfoundation.org/pipermail/squeak-dev/2004-September/082071.html"  Float negativeZero.  Float pi.  101.10001 }! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:33'!
floats

	| temp |
	temp := 1.1.
	^ (1 to: 1 << 15) collect: [ :i | temp := temp negated. i * temp ]
! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
fractionSamples

	^Array with: 3/4 with: 0-(1001/1000)! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/6/2010 18:10'!
fromSelector: aSelector

	^ self new 
		title: aSelector asString;
		content: (self perform: aSelector); 
		yourself! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/15/2011 15:30'!
fromSelectors: aCollectionOfSampleSelectors
	"Answer a collection of instances of me, with the selectors. Each selector should define a class-side method."

	^aCollectionOfSampleSelectors
		collect: [ :aSelector | self fromSelector: aSelector ]! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MartinDias 12/30/2011 19:50'!
globalSamples

	^{ Integer.  Integer class }! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MartinDias 9/19/2011 15:24'!
globals

	| classes associations sample |	
	classes := Smalltalk globals allClasses first: 500.
	associations := classes collect: [:aClass | Smalltalk globals associationAt: aClass name].
	sample := OrderedCollection new.
	100 timesRepeat: [ sample addAll: classes. sample addAll: associations. ].
	^sample! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive samples' stamp: 'MartinDias 1/7/2012 14:32'!
hookSubstitutions

	^ (1 to: 1 << 14) 
		collect: [ :i | FLHookSubstitutedObject with: i ].
! !

!FLBenchmarkSample classSide methodsFor: 'private references samples' stamp: 'MartinDias 1/1/2012 20:21'!
indexedWithOneByte
	"Answer an object that is serialized using 1 byte indexes."
	
	^ self points: 5! !

!FLBenchmarkSample classSide methodsFor: 'private references samples' stamp: 'MartinDias 1/1/2012 20:22'!
indexedWithThreeBytes
	"Answer an object that is serialized using 3 byte indexes."

	^ self points: 17! !

!FLBenchmarkSample classSide methodsFor: 'private references samples' stamp: 'MartinDias 1/1/2012 20:22'!
indexedWithTwoBytes
	"Answer an object that is serialized using 2 byte indexes."

	^ self points: 11! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
largeIntegerSamples

	^Array
		with: 100 factorial
		with: (0-100 factorial)
! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/3/2011 16:36'!
largeIntegers
	| numbers |
	numbers := OrderedCollection new.
	numbers addAll:  ( 1 << 30 to:( ( 1 << 30) + 10000) ) asArray.
	numbers addAll: ((( 1 << 30 to:( ( 1 << 30) + 10000) ) asArray ) collect: [:each | each negated]).
	^ numbers
		

! !

!FLBenchmarkSample classSide methodsFor: 'private helpers' stamp: ''!
largePoints
	^ (1 to: 1 << 16) 
		collect: [ :i | i @ (i+1) ]! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
metaClassSamples

	^ self classSamples collect: [ : each | each class ]! !

!FLBenchmarkSample classSide methodsFor: 'private advanced samples' stamp: 'MarianoMartinezPeck 5/27/2011 12:08'!
methodDictionaries
	| methodDicts |
	methodDicts := OrderedCollection new.
	methodDicts addAll: ((SystemNavigation default allClassesInPackageNamed: 'System') 
		collect: [ :aClass | aClass methodDict ]).
	methodDicts addAll: ((SystemNavigation default allClassesInPackageNamed: 'Morphic') 
		collect: [ :aClass | aClass methodDict ]).
	^ methodDicts! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 10:09'!
methodsInCategory: aString do: twoArgBlock

	(self class methodsInCategory: aString) do:
		[ : each |
		twoArgBlock
			value: each
			value: (self perform: each) ]! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MartinDias 8/30/2011 01:29'!
nilAndBooleans
	
	^ (1 to: 1 << 17) 
		collect: [ :index | #(nil true false) at: 1 + (index \\ 3) ]! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 16:34'!
nonPrimitiveSampleSelectors
			
	^ (self class methodsInCategory: 'private non-primitive samples'),  (self class methodsInCategory: 'private non-primitive collection samples')! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/15/2011 15:31'!
nonPrimitiveSamples

	^self fromSelectors: self nonPrimitiveSampleSelectors! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive collection samples' stamp: 'MarianoMartinezPeck 10/18/2011 16:07'!
orderedCollectionSamples

	| all |
	all := OrderedCollection new.
	30 timesRepeat: [all add: self allSmallSamples asOrderedCollection].
	^ all asArray
	! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:33'!
pairs

	^ (1 to: 1 << 14) 
		collect: [ :i | 
			FLPair new 
				left: i; 
				right: i+1;
				yourself ].
! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive samples' stamp: 'MartinDias 12/30/2011 16:06'!
points

	^ self points: 14! !

!FLBenchmarkSample classSide methodsFor: 'private helpers' stamp: 'MartinDias 12/30/2011 16:03'!
points: exponent

	^ (1 to: 1 << exponent) 
		collect: [ :i | i @ (i+1) ]! !

!FLBenchmarkSample classSide methodsFor: 'private helpers' stamp: 'MarianoMartinezPeck 9/6/2011 23:17'!
populatedDictionary

	| populatedDictionary counter |
	
	populatedDictionary := Dictionary new.
	counter := 0.
	100 timesRepeat:
		[ counter := counter + 1.
		populatedDictionary
			at: counter
			put: self allSmallSamples ].
	^ populatedDictionary! !

!FLBenchmarkSample classSide methodsFor: 'private advanced samples' stamp: 'MarianoMartinezPeck 5/17/2011 10:27'!
primitiveMethods
	| methods |
	methods := OrderedCollection new.
	methods addAll: SystemNavigation default allPrimitiveMethods.
	methods addAll: SystemNavigation default allPrimitiveMethods.
	methods addAll: SystemNavigation default allPrimitiveMethods.
	^ methods! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 16:39'!
primitiveSampleSelectors
			
	^ self class methodsInCategory: 'private primitive samples'! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/15/2011 15:31'!
primitiveSamples

	^self fromSelectors: self primitiveSampleSelectors! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
processSamples
	"Processes are not supported by Ma object serialization at this time."
	^ { 
		nil.
	 }! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
readStreamSamples

	^Array
		with: (ReadStream on: (1 to: 1000));
		with: (ReadStream on: #('chris' 'paula'))
! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
readWriteStreamSamples

	^Array
		with: (ReadWriteStream on: (1 to: 1000));
		with: (ReadWriteStream on: #('chris' 'paula'))
! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive samples' stamp: 'MartinDias 12/6/2010 18:52'!
rectangles
	
	^ self rectanglesBy: 4! !

!FLBenchmarkSample classSide methodsFor: 'private helpers' stamp: 'MarianoMartinezPeck 6/9/2011 23:41'!
rectanglesBy: step
	"Returns rectangles sharing more or less integers, depending step is 1, 2, 3 or 4."

	^ (1 to: (2 raisedTo: 12) * step by: step) 
		collect: [ :i | i@(i+1) corner: (i+2)@(i+3) ]
! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
scaledDecimalSamples
	^ { 
		((13 / 11) asScaledDecimal: 6).
		((11 / 13) asFloat asScaledDecimal)
	 }! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:41'!
scaledDecimals

	| temp |
	temp := 17347.1375656s6.
	^ (1 to: 1 << 12) collect: [ :i | temp := temp negated. i / temp ]
! !

!FLBenchmarkSample classSide methodsFor: 'private non-primitive collection samples' stamp: 'MarianoMartinezPeck 10/18/2011 16:07'!
setSamples
	| all |
	all := OrderedCollection new.
	30 timesRepeat: [ all add: (Set new addAll: (self allSmallSamples copyWithout: nil); yourself) ].
	^ all asArray! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
smallIntegerSamples

	^Array
		with: -1212
		with: -7
		with: 0
		with: 1111
! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:34'!
smallIntegers
	| numbers |
	numbers :=  OrderedCollection new.
	numbers addAll: (1 to: 1 << 29 by: 1 << 14) asArray.
	numbers addAll: ((1 to: 1 << 29 by: 1 << 14) asArray collect: [:each | each negated] ).
	^ numbers	
	
! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 9/9/2011 22:26'!
smallIntegers16Bits
	| numbers |
	numbers :=  OrderedCollection new.
	numbers addAll: (1 to: 1 << 14) asArray.
	numbers addAll: ((1 to: 1 << 14) asArray collect: [:each | each negated] ).
	^ numbers	
	
! !

!FLBenchmarkSample classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/10/2011 16:37'!
smallSamplesDo: twoArgBlock

	self
		methodsInCategory: 'private small samples'
		do: twoArgBlock! !

!FLBenchmarkSample classSide methodsFor: 'private advanced samples' stamp: 'MarianoMartinezPeck 5/17/2011 10:26'!
someMethods
	| methods |
	methods := OrderedCollection new.
	methods addAll: ((SystemNavigation default allClassesInPackageNamed: 'System') 
		inject: Set new into: [ :aSet :aClass | aSet addAll: aClass methods. aSet ]).
	methods addAll: ((SystemNavigation default allClassesInPackageNamed: 'Morphic') 
		inject: Set new into: [ :aSet :aClass | aSet addAll: aClass methods. aSet ]).
	^ methods! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
soundSamples
| classGetter |
classGetter := [ : name |  (Smalltalk classNamed: #PluckedSound) ifNotNilDo: [ : theClass | theClass default ]. ].
^#(PluckedSound Speaker)  collect:
[ : eachClassName |
	classGetter value: eachClassName ]! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
stringSamples

	| allCharactersString |

	allCharactersString := (0 to: 255) asByteArray asString.
	^Array
		with: String new
		with: 'hello'
		with: allCharactersString
! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 23:20'!
strings
	
	^ (1 to: 1 << 15) 
		collect: [ :index | (index * 123456790) asString ]! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
symbolSamples

	^Array
		with: #size
		with: #with:with:with:with:
		with: #'hello there'
		"with: '' asSymbol  <- conflicts with empty String sample since, in Squeak, they're equal.."
! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/10/2011 16:49'!
symbols
	
	^ (1 to: 1 << 15) 
		collect: [ :index | (index * 123456790) asString asSymbol ]! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 9/1/2011 21:09'!
times
	| times initialTime |
	times := OrderedCollection new.
	initialTime := Time fromSeconds: 76020.
	1 to: 30000 do: [:index |
		times add: (initialTime addSeconds: index).
		].
	^ times! !

!FLBenchmarkSample classSide methodsFor: 'instance creation' stamp: 'MartinDias 12/7/2010 10:09'!
title: aTitle content: aContent

	^ self new
		title: aTitle;
		content: aContent;
		yourself! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
uuidSamples

	^ { UUID fromString: 'a3b64357-377a-5b41-b575-1c653084a121'.
	UUID nilUUID }! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
wideStringSamples
	^ Array with: (((12354 to: 12362) collect: [ : each | Unicode value: each ]) as: String)! !

!FLBenchmarkSample classSide methodsFor: 'private primitive samples' stamp: 'MarianoMartinezPeck 6/9/2011 15:01'!
wideStrings
	| all |
	all := OrderedCollection new.
	600 timesRepeat: [
	       all add: (WideString
	               streamContents: [ :stream |
	                       600 timesRepeat: [
	                               stream nextPut: (256 to: 1000) atRandom asCharacter ] ] ) ].
	^ all! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:48'!
writeStreamSamples

	^Array
		with: (WriteStream on: String new);
		with: (WriteStream on: (OrderedCollection new: 200))
! !

!FLBenchmarkSample classSide methodsFor: 'private small samples' stamp: 'MarianoMartinezPeck 6/9/2011 17:49'!
zipCodes

	^#(
		'64114'
		'64131'
		'66101'
		'10201'
		'98876'
		'12345'
		'71717')! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
content
	^ content! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 6/9/2011 20:49'!
content: anObject
	content := anObject! !

!FLBenchmarkSample methodsFor: 'printing' stamp: 'MartinDias 8/9/2011 15:06'!
printOn: aStream
	"Append a sequence of characters to aStream that identify the receiver."

	super printOn: aStream.
	aStream
		nextPutAll: ' title: ';
		print: title! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
title
	^ title! !

!FLBenchmarkSample methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:01'!
title: anObject
	title := anObject! !

Object subclass: #FLBenchmarkTotalResultForRunner
	instanceVariableNames: 'runner totalTimeSerialization totalTimeMaterialization'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarkTotalResultForRunner classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 5/17/2011 12:06'!
runner: aRunner totalTimeSerialization: millisecondsSerialization totalTimeMaterialization: millisecondsMatarialization
	^ self new initializeWithRunner: aRunner totalTimeSerialization: millisecondsSerialization totalTimeMaterialization: millisecondsMatarialization! !

!FLBenchmarkTotalResultForRunner methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/17/2011 12:07'!
initializeWithRunner: aRunner totalTimeSerialization: millisecondsSerialization totalTimeMaterialization: millisecondsMaterialization
	runner := aRunner. 
	totalTimeSerialization := millisecondsSerialization.
	totalTimeMaterialization := millisecondsMaterialization.! !

!FLBenchmarkTotalResultForRunner methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 21:57'!
printResultsOn: aStream
	aStream cr.
	aStream nextPutAll:  runner runnerName, ' total milliseconds for serialization: ', totalTimeSerialization asString; cr.
	aStream nextPutAll:  runner runnerName, ' total milliseconds for materialization: ', totalTimeMaterialization asString; cr.
	aStream nextPutAll:  runner runnerName, ' total milliseconds: ', (totalTimeMaterialization +  totalTimeSerialization) asString; cr.
! !

!FLBenchmarkTotalResultForRunner methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 21:57'!
printResultsOn: aStream comparingToFuelResults: fuelResults
	| dif |

	self printResultsOn:  aStream.
	(fuelResults isNil or: [runner isFuelRunner]) ifTrue: [^ self ].
	aStream cr.
		(totalTimeSerialization > fuelResults totalTimeSerialization) 
			ifTrue: [
				dif := (totalTimeSerialization / fuelResults totalTimeSerialization) asFloat .
				aStream nextPutAll: 'For serialization, ', fuelResults runner runnerName, ' is ', (dif printShowingDecimalPlaces: 2), ' times faster than ', runner runnerName; cr.
			] ifFalse: [
				dif := ( fuelResults totalTimeSerialization / totalTimeSerialization) asFloat .
				aStream nextPutAll: 'For serialization, ', fuelResults runner runnerName, ' is ', (dif printShowingDecimalPlaces: 2), ' times slower than ', runner runnerName; cr.
			].
			
			(totalTimeMaterialization  > fuelResults totalTimeMaterialization ) 
			ifTrue: [
				dif := (totalTimeMaterialization / fuelResults totalTimeMaterialization) asFloat .
				aStream nextPutAll: 'For materialization, ', fuelResults runner runnerName, ' is ', (dif printShowingDecimalPlaces: 2), ' times faster than ', runner runnerName; cr.
			] ifFalse: [
				dif := ( fuelResults totalTimeMaterialization / totalTimeMaterialization) asFloat .
				aStream nextPutAll: 'For materialization, ', fuelResults runner runnerName, ' is ', (dif printShowingDecimalPlaces: 2), ' times slower than ', runner runnerName; cr.
			]
			

! !

!FLBenchmarkTotalResultForRunner methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 12:25'!
runner
	^ runner! !

!FLBenchmarkTotalResultForRunner methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 12:25'!
totalTimeMaterialization
	^ totalTimeMaterialization! !

!FLBenchmarkTotalResultForRunner methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/17/2011 12:25'!
totalTimeSerialization
	^ totalTimeSerialization! !

Object subclass: #FLBenchmarks
	instanceVariableNames: 'results samples runners writeStreamFactoryBlock readStreamFactoryBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarks commentStamp: 'MartinDias 10/6/2011 03:04' prior: 0!
I represent benchmarks of Fuel against other serializers. 

You can see in FLBenchmarks class>>newSmall a tiny example of benchmark creation. Either FLBenchmarks>>useFileStream or FLBenchmarks>>useMemoryStream indicates in which stream serialize. After running, you should use the 'printing' protocol to show results, usually on Transcript or in a CSV file.

Following, some useful examples:


- Standard set of benchmarks:

FLBenchmarks newBasic run printResultsOnTranscript.


- A more complete sample set but just with Fuel (useful for compare between Fuel versions):

FLBenchmarks newFullFuelOnly run printResultsOnTranscript.


- Customized sample set, just with Fuel (useful for compare between Fuel versions):

(FLBenchmarks newFuelOnlySamplesFrom: #(#strings #symbols)) run printResultsOnTranscript.


- This exports sizes of the serialized streams:

FLBenchmarks newBasic run printSizeResultsOnCSVNamed: 'sizes.csv'.


- To see how much ImageSegment gets slow when shared objects increase:

FLBenchmarks generateSharedSamplesCSV.


- To see how time evolves when a sample increases its size:

FLBenchmarks generateSubsetSamplesCSV.

- A much more complex examples is the following:

FLBenchmarks new
	samplesSelectors: #( #strings #symbols );
	runners: {FLFuelBenchmarkRunner new};
	useMemoryStream;
	run;
	printResultsOnTranscript!

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 12/9/2011 15:23'!
benchFuelGZipped
	FLBenchmarks new 
		samples: FLBenchmarkSample basicSamples;
		runners: (Array with: FLFuelGZipBenchmarkRunner new with: FLFuelBenchmarkRunner new);
		useFileStream;
		run;
		printResultsOnTranscript.! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MarianoMartinezPeck 6/18/2011 19:33'!
generateAllCSVFilesForPaper
	self newPrimitiveForPaperInMemory run printResultsOnCSVNamed: 'BenchForPrimitiveObjectsSerializedInMemory.csv'. 
	self newNonPrimitiveForPaperInMemory run printResultsOnCSVNamed: 'BenchForNonPrimitiveObjectsSerializedInMemory.csv'.
	self newPrimitiveForPaperInFile run printResultsOnCSVNamed: 'BenchForPrimitiveObjectsSerializedInFile.csv'.
	self newNonPrimitiveForPaperInFile run printResultsOnCSVNamed: 'BenchForNonPrimitiveObjectsSerializedInFile.csv'.
	self newPrimitiveForPaperInFile run printSizeResultsOnCSVNamed: 'BenchStreamSize.csv'.

! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 8/13/2011 18:29'!
generateFuelMetacelloVersionsComparison

	| fuelVersions repetitionNumber runs |
	fuelVersions := #('1.5' '1.6').
	repetitionNumber := 2.
	FileStream
		forceNewFileNamed: 'benchMetacelloVersionsComparison.txt'
		do: [ :benchsFile | 
			"generate runs"
			runs := OrderedCollection new.
			repetitionNumber timesRepeat: [ runs addAll: fuelVersions ].

			"run"
			runs
				do: [ :fuelVersion | 
					(ConfigurationOfFuel project version: fuelVersion) load.
					FuelUtilities cleanUpEverything.
					benchsFile
						nextPutAll: ' === ' , fuelVersion , ' === ';
						cr.
					3 timesRepeat: [ Smalltalk garbageCollect ].
					FLBenchmarks new
						samples: FLBenchmarkSample basicSamples;
						runners: {FLFuelBenchmarkRunner new};
						useFileStream;
						run;
						printResultsOn: benchsFile ]
				displayingProgress: [ :fuelVersion | 'Benchmarks batch running version ' , fuelVersion ] ].
! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 8/8/2011 15:59'!
generateFuelVersionsComparison

	| fuelVersions repetitionNumber runs |
	fuelVersions := #('Fuel-MartinDias.305' 'Fuel-MartinDias.306').
	repetitionNumber := 2.
	FileStream
		forceNewFileNamed: 'benchsClusterizationReificationBis.txt'
		do: [ :benchsFile | 
			"generate runs"
			runs := OrderedCollection new.
			repetitionNumber timesRepeat: [ runs addAll: fuelVersions ].

			"run"
			runs
				do: [ :fuelVersion | 
					(Gofer new)
						version: fuelVersion;
						load.
					FuelUtilities cleanUpEverything.
					benchsFile
						nextPutAll: ' === ' , fuelVersion , ' === ';
						cr.
					3 timesRepeat: [ Smalltalk garbageCollect ].
					FLBenchmarks new
						samples: FLBenchmarkSample basicSamples;
						runners: {FLFuelBenchmarkRunner new};
						useFileStream;
						run;
						printResultsOn: benchsFile ]
				displayingProgress: [ :fuelVersion | 'Benchmarks batch running version ' , fuelVersion ] ]! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 12/16/2011 01:00'!
generateMiniFullSerializationVersionsComparison


	| versions repetitionNumber runs anObject time |
	versions := #('FuelMetalevel-MarianoMartinezPeck.10' 'FuelMetalevel-MartinDias.13').
	repetitionNumber := 2.
	"anObject := ColorSelectorDialogWindow."
	anObject := nil.
	
	FileStream
		forceNewFileNamed:  'unifiedGlobals4-micro.txt'
		do: [ :benchsFile | 
			"generate runs"
			runs := OrderedCollection new.
			repetitionNumber timesRepeat: [ runs addAll: versions ].

			"run"
			runs
				do: [ :fuelVersion | 
					(Gofer new)
						version: fuelVersion;
						load.
					FuelUtilities cleanUpEverything.

					benchsFile nextPutAll: ' === ' , fuelVersion , ' === '; cr.
					
					3 timesRepeat: [ Smalltalk garbageCollect ].

					time := [ | s |
						s := FLSerializer newFull.
						"s analyzer considerInternalAll: {anObject}."
						s serialize: anObject on: (ByteArray new: 100) writeStream ] bench.
					
					benchsFile nextPutAll: time printString; cr. 
						
				] displayingProgress: [ :fuelVersion | 'Benchmarks batch running version ' , fuelVersion ] ]! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MartinDias 2/17/2012 13:46'!
generateMiniSerializationVersionsComparison

	| versions repetitionNumber runs anObject time stream |
	versions := #('Fuel-MartinDias.622' 'Fuel-MarianoMartinezPeck.618').
	repetitionNumber := 2.
	anObject := (FLBenchmarkSample fromSelector: #zipCodes) content.
	
	FileStream
		forceNewFileNamed: 'benchsMicroLatestChanges.txt'
		do: [ :benchsFile | 
			"generate runs"
			runs := OrderedCollection new.
			repetitionNumber timesRepeat: [ runs addAll: versions ].

			"run"
			runs
				do: [ :fuelVersion | 
					(Gofer new)
						version: fuelVersion;
						load.
					FuelUtilities cleanUpEverything.

					benchsFile nextPutAll: ' === ' , fuelVersion , ' === '; cr.
					
					3 timesRepeat: [ Smalltalk garbageCollect ].

					time := [ 
						stream := (ByteArray new: 100) writeStream.
						FLSerializer newDefault 
						serialize: anObject 
						on: stream ] bench.
					
					benchsFile 
						nextPutAll: 'size: ';
						nextPutAll: stream contents size printString; 
						cr;
						nextPutAll: 'bench: ';
						nextPutAll: time printString; 
						cr. 
						
				] displayingProgress: [ :fuelVersion | 'Benchmarks batch running version ' , fuelVersion ] ]! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MarianoMartinezPeck 6/13/2011 12:23'!
generateSharedSamplesCSV

   	| samples |
	samples := OrderedCollection new.

	0 to: 100 by: 20 do: [ :percent |
		| sample subsetSize aSubsetOfSample |
		sample := FLBenchmarkSample fromSelector: #points.
		sample title: (percent asString) , $% asString.  
		samples add: sample.
       
		subsetSize := (sample content size * percent) // 100.
		aSubsetOfSample := sample content asOrderedCollection first: subsetSize.
		Smalltalk globals at: ('FuelBenchmarksShared', percent printString) asSymbol put: aSubsetOfSample ].
   
	FLBenchmarks new
		samples: samples;
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. };
		useMemoryStream;
		run;
		printResultsOnCSVNamed: 'imageSegmentSharedObjectsBenchmarks.csv'. . ! !

!FLBenchmarks classSide methodsFor: 'scripts' stamp: 'MarianoMartinezPeck 6/13/2011 12:51'!
generateSubsetSamplesCSV

   	| samples |
	samples := OrderedCollection new.

	0 to: 100 by: 10 do: [ :percent |
		| sample subsetSize aSubsetOfSample |
		sample := FLBenchmarkSample fromSelector: #points.
		subsetSize := (sample content size * percent) // 100.
		aSubsetOfSample := sample content asOrderedCollection first: subsetSize. 
		
		sample title: (percent asString) , $% asString.
		sample content: aSubsetOfSample.
		samples add: sample].
   
	FLBenchmarks new
		samples: samples;
		runners: (Array with: FLFuelBenchmarkRunner new with: FLSmartRefStreamBenchmarkRunner new with: FLImageSegmentBenchmarkRunner new with: FLMagmaBenchmarkRunner new);
		useMemoryStream;
		run;
		printResultsOnCSVNamed: 'subsetBenchmarks.csv'. ! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/1/2011 22:19'!
newBasic

	^ self new 
		samples: FLBenchmarkSample basicSamples;
		runners: FLBenchmarkRunner allRunners;
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/10/2011 17:12'!
newBasicForPaperInFile

	^ self new 
		samples: (FLBenchmarkSample basicSamples);
		runners: (Array with: FLFuelBenchmarkRunner new with: FLSmartRefStreamBenchmarkRunner new with: FLImageSegmentBenchmarkRunner new with: FLMagmaBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/17/2011 16:52'!
newBasicForPaperInMemory

	^ self new 
		samples: (FLBenchmarkSample basicSamples);
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. FLMagmaBenchmarkRunner new. };
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/10/2011 10:49'!
newBasicFuelOnly

	^ self new 
		samples: FLBenchmarkSample basicSamples;
		runners: (Array with: FLFuelBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/9/2011 15:06'!
newBasicImageSegmentOnlyInFile

	^ self new 
		samples: FLBenchmarkSample basicSamples;
		runners: (Array with: FLImageSegmentBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/9/2011 15:06'!
newBasicImageSegmentOnlyInMemory

	^ self new 
		samples: FLBenchmarkSample basicSamples;
		runners: (Array with: FLImageSegmentBenchmarkRunner new);
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/1/2011 22:20'!
newBasicSmartRefStreamOnly

	^ self new 
		samples: FLBenchmarkSample basicSamples;
		runners: (Array with: FLSmartRefStreamBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/9/2011 21:06'!
newBasicSmartRefStreamOnlyInMemory

	^ self new 
		samples: FLBenchmarkSample basicSamples;
		runners: (Array with: FLSmartRefStreamBenchmarkRunner new);
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/17/2011 10:50'!
newCogVSNonCoglOnly

	^ self new 
		samples: FLBenchmarkSample primitiveSamples;
		runners: (Array with: FLFuelBenchmarkRunner new with: FLImageSegmentBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/18/2011 15:24'!
newFuelOnlyNonPrimitive

	^ self new 
		samples: (FLBenchmarkSample nonPrimitiveSamples);
		runners: { FLFuelBenchmarkRunner new. };	
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 9/1/2011 20:08'!
newFuelOnlySamplesFrom: aCollectionOfSampleSelectors

	^ self new 
		samplesSelectors: aCollectionOfSampleSelectors;
		runners: (Array with: FLFuelBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/1/2011 22:20'!
newFullFuelOnly

	^ self new 
		samples: FLBenchmarkSample allSamples;
		runners: (Array with: FLFuelBenchmarkRunner new);
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/17/2011 17:07'!
newNonPrimitiveForPaperInFile

	^ self new 
		samples: (FLBenchmarkSample nonPrimitiveSamples);
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. FLMagmaBenchmarkRunner new.  FLStompBenchmarkRunner new. FLSRPBenchmarkRunner new. FLEsAndEmBenchmarkRunner new };	
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/17/2011 17:07'!
newNonPrimitiveForPaperInMemory

	^ self new 
		samples: (FLBenchmarkSample nonPrimitiveSamples);
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. FLMagmaBenchmarkRunner new.  FLStompBenchmarkRunner new. FLSRPBenchmarkRunner new. FLEsAndEmBenchmarkRunner new };	
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/17/2011 17:07'!
newPrimitiveForPaperInFile

	^ self new 
		samples: (FLBenchmarkSample primitiveSamples);
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. FLMagmaBenchmarkRunner new.  FLStompBenchmarkRunner new. FLSRPBenchmarkRunner new. FLEsAndEmBenchmarkRunner new };	
		useFileStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 6/17/2011 16:53'!
newPrimitiveForPaperInMemory

	^ self new 
		samples: (FLBenchmarkSample primitiveSamples);
		runners: { FLFuelBenchmarkRunner new. FLSmartRefStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. FLMagmaBenchmarkRunner new.  FLStompBenchmarkRunner new. FLSRPBenchmarkRunner new. FLEsAndEmBenchmarkRunner new };	
		useMemoryStream;
		yourself.! !

!FLBenchmarks classSide methodsFor: 'instance creation' stamp: 'mariano 6/3/2011 11:22'!
newSmall

	^ self new 
		samples: { FLBenchmarkSample fromSelector: #points. FLBenchmarkSample fromSelector: #strings };
		runners: { FLFuelBenchmarkRunner new. FLReferenceStreamBenchmarkRunner new. FLImageSegmentBenchmarkRunner new. };
		useFileStream;
		yourself.! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 10/6/2011 01:24'!
addResultRunning: aBlockClosure titled: aTitle run: aRun typeOfOperation: aSymbol streamSize: aNumberOfBytes

	| timeToRun successful |

	"In CogVM the first time a method is used, it is executed in the old slow way, that is, interpreted, and it is added to the cache.. The second time a method is used, that means, when it is found in the cache, Cog converts that method to machine code. However, to do that, extra time is needed. Only the third time the method will be executed as machine code and without extra effort. So, for the samples we run 2 times the same sample so that to be sure to be always in the same condition."
	"A GC can occur at any time and we cannot control it. However, doing a GC before the probe, we minimize the chances of having a GC during the probe and put noise in the measure because of the GC time."
	[ 2 timesRepeat: [ aBlockClosure value ]. 
	3 timesRepeat: [ Smalltalk garbageCollect ].
	timeToRun := Time millisecondsToRunWithoutContextSwitch: aBlockClosure. 
	successful := true ] 
		on: Error
		do: [ timeToRun := 0. successful := false ].

	results add: (FLBenchmarkResult new
		title: aTitle;
		run: aRun;
		milliseconds: timeToRun;
		successful: successful;
		typeOfOperation: aSymbol;
		serializationSize: aNumberOfBytes;
		yourself)! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 22:46'!
addResultRunningMaterialization: aBlockClosure titled: aTitle run: aRun serializationSize: aNumberOfBytes

	^ self addResultRunning: aBlockClosure titled: aTitle run: aRun typeOfOperation: #materialization streamSize: aNumberOfBytes
! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 19:42'!
addResultRunningSerialization: aBlockClosure titled: aTitle run: aRun

	^ self addResultRunning: aBlockClosure titled: aTitle run: aRun typeOfOperation: #serialization streamSize: 0
! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 7/1/2011 14:11'!
initialize 

	super initialize.
	results := OrderedCollection new.
	runners := OrderedCollection new.
	samples := OrderedCollection new.! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:46'!
printResultsOn: aStream

	(FLBenchmarksClassicPrinter on: aStream) print: self
! !

!FLBenchmarks methodsFor: 'printing' stamp: 'mariano 6/3/2011 10:07'!
printResultsOnCSV

	FileStream 
		forceNewFileNamed: 'benchmarks.csv'
		do: [ :aFile | self printResultsOnCSV: aFile ]
! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 19:44'!
printResultsOnCSV: aStream

	(FLBenchmarksCSVPrinter on: aStream) print: self! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/17/2011 19:04'!
printResultsOnCSVNamed: aFileName

	FileDirectory default 
		forceNewFileNamed: aFileName 
		do: [ :aFile | (FLBenchmarksCSVPrinter on: aFile) print: self ]! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:53'!
printResultsOnTranscript

	self printResultsOn: Transcript.
	Transcript flush.! !

!FLBenchmarks methodsFor: 'printing' stamp: 'MartinDias 6/17/2011 22:19'!
printSizeResultsOnCSVNamed: aFileName

	FileDirectory default 
		forceNewFileNamed: aFileName 
		do: [ :aFile | (FLBenchmarksSizeCSVPrinter on: aFile) print: self ]! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/5/2011 16:07'!
readStreamNamed: aName

	^ readStreamFactoryBlock value: aName! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 00:39'!
results

	^ results! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
results: anObject
	results := anObject! !

!FLBenchmarks methodsFor: 'running' stamp: 'MartinDias 6/2/2011 19:16'!
run

	self runners 
		do: [ :aRunner | self run: aRunner ] 
		displayingProgress: [ :aRunner | 'Running benchmarks for ', aRunner runnerName ]! !

!FLBenchmarks methodsFor: 'running' stamp: 'MartinDias 6/2/2011 19:09'!
run: aRunner

	(self runsForRunner: aRunner) 
		do: [ :aRun | aRun runOn: self ] 
		displayingProgress: [ :aRun | aRun sample title ]
! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:19'!
runners

	^ runners! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
runners: anObject
	runners := anObject! !

!FLBenchmarks methodsFor: 'protected' stamp: 'mariano 5/31/2011 16:30'!
runsForRunner: aRunner

	| runs |
	runs := OrderedCollection new.
	self samples do: [ :aSample | 
			runs add: 
			(FLBenchmarkRun 
				sample: aSample
				runner: aRunner).
			 ].
	^ runs.

! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
samples
	^ samples! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MartinDias 12/6/2010 18:26'!
samples: anObject
	samples := anObject! !

!FLBenchmarks methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/1/2011 20:04'!
samplesSelectors: sampleSelectors
	samples := FLBenchmarkSample fromSelectors: sampleSelectors! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/5/2011 16:00'!
useFileStream
	
	writeStreamFactoryBlock := [ :aName | (FileDirectory default forceNewFileNamed:  'BenchFor', aName) ].
	readStreamFactoryBlock := [ :aName | (FileDirectory default oldFileNamed:  'BenchFor', aName) ].
	"We do not set here the binary mode becuse some serializers like SIXX are text based. Each serializer that needs to have binary streams should send #binary in the methods #serialize:on: and #meterializeOn: of its runner class"! !

!FLBenchmarks methodsFor: 'initialize-release' stamp: 'MartinDias 10/5/2011 16:19'!
useMemoryStream
	"Don't use RWBinaryOrTextStream because it doesn't work with ImageSegment"

	| inMemoryStream |
	writeStreamFactoryBlock := [ :aName | inMemoryStream :=  (MultiByteBinaryOrTextStream on: '' ) ].
	readStreamFactoryBlock := [ :aName | inMemoryStream reset ]
	
	"We do not set here the binary mode becuse some serializers like SIXX are text based. Each serializer that needs to have binary streams should send #binary in the methods #serialize:on: and #meterializeOn: of its runner class"! !

!FLBenchmarks methodsFor: 'protected' stamp: 'MarianoMartinezPeck 10/5/2011 16:07'!
writeStreamNamed: aName

	^ writeStreamFactoryBlock value: aName! !

Object subclass: #FLBenchmarksCSVPrinter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarksCSVPrinter commentStamp: 'MartinDias 6/2/2011 20:48' prior: 0!
My goal is export benchmark results to a CSV file. Useful for creating a chart with tools like OpenOffice or Excel.

An example of use:

FLBenchmarks 
	newBasic
	run; 
	printResultsOnCSV.!

!FLBenchmarksCSVPrinter classSide methodsFor: 'instance creation' stamp: 'MartinDias 11/11/2010 18:01'!
on: aStream

	^ self basicNew 
		initialize;
		stream: aStream;
		yourself! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/2/2011 17:17'!
columnNamesFrom: benchmarkResultsGroupedByRow
	
	^ (self columnOrderedResults: benchmarkResultsGroupedByRow anyOne) 
		collect: [ :aResult | aResult run sample title ]! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/2/2011 17:17'!
columnOrderedResults: results
	
	^ results sort: [ :x :y | (x run sample title) <= (y run sample title) ]! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 18:56'!
dataFrom: aResult

	^ aResult milliseconds printString! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/2/2011 17:19'!
nextPutColumnNameRowOf: aBenchmarkResults
	"Writes the column names row"

	stream nextPutAll: 'run name'.
	(self columnNamesFrom: aBenchmarkResults)
		do: [ :aColumnName | stream nextPutAll: ', ', aColumnName ].
	stream cr.! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'mariano 6/3/2011 11:19'!
nextPutDataRowsOf: benchmarkResultsGroupedByRow
	"Writes the data rows"
	
	self nextPutSerializationDataRowsOf: benchmarkResultsGroupedByRow.
	self nextPutMaterializationDataRowsOf: benchmarkResultsGroupedByRow.

! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'mariano 6/3/2011 11:21'!
nextPutMaterializationDataRowsOf: benchmarkResultsGroupedByRow
	
	self nextPutOperation: 'Materialization' dataRowsOf: benchmarkResultsGroupedByRow
	

! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 18:56'!
nextPutOperation: anOperationName dataRowsOf: benchmarkResultsGroupedByRow
	| titles resultsInRow |
	
	stream cr.
	self nextPutColumnNameRowOf: benchmarkResultsGroupedByRow.
	
	titles := (benchmarkResultsGroupedByRow keys select: [:each | each includesSubString: anOperationName ]) sort: [:a :b | a < b ].
	
	titles do: [:title |

		"Grouping attribute"
		stream nextPutAll: title.
		
		resultsInRow := benchmarkResultsGroupedByRow at: title.
		"Data to show in cells"
		(self columnOrderedResults: resultsInRow) do: [ :aResult | 
			stream nextPutAll: ', ', (self dataFrom: aResult) ].
		stream cr.
	].
	stream cr ! !

!FLBenchmarksCSVPrinter methodsFor: 'protected' stamp: 'mariano 6/3/2011 11:21'!
nextPutSerializationDataRowsOf: benchmarkResultsGroupedByRow
	
	self nextPutOperation: 'Serialization' dataRowsOf: benchmarkResultsGroupedByRow
	
! !

!FLBenchmarksCSVPrinter methodsFor: 'printing' stamp: 'mariano 6/3/2011 11:24'!
print: benchmarks

	| resultsGroupedByRow |

	resultsGroupedByRow := benchmarks results groupedBy: [ :x | x title ].
		
	self nextPutDataRowsOf: resultsGroupedByRow.! !

!FLBenchmarksCSVPrinter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:36'!
stream

	^ stream! !

!FLBenchmarksCSVPrinter methodsFor: 'accessing' stamp: 'MartinDias 11/4/2010 10:36'!
stream: anObject

	stream := anObject! !

FLBenchmarksCSVPrinter subclass: #FLBenchmarksSizeCSVPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarksSizeCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 22:37'!
dataFrom: aResult

	^ aResult serializationSize printString! !

!FLBenchmarksSizeCSVPrinter methodsFor: 'protected' stamp: 'MartinDias 6/17/2011 18:53'!
nextPutDataRowsOf: benchmarkResultsGroupedByRow
	"Writes the data rows"
	
	self nextPutMaterializationDataRowsOf: benchmarkResultsGroupedByRow.

! !

Object subclass: #FLBenchmarksClassicPrinter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLBenchmarksClassicPrinter commentStamp: 'MartinDias 6/2/2011 20:54' prior: 0!
My goal is export benchmark results in a human readable format.

An example of use:

FLBenchmarks 
	newBasic
	run; 
	printResultsOnTranscript

!

!FLBenchmarksClassicPrinter classSide methodsFor: 'instance creation' stamp: 'MartinDias 6/2/2011 19:54'!
on: aStream

	^ self basicNew 
		initialize;
		stream: aStream;
		yourself! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:53'!
print: benchmarks 

	self printIndividualResults: benchmarks results.
	self printTotals: benchmarks results.! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 23:01'!
printIndividualResults: results

	results do: [ :aResult | 
			aResult printOn: stream.
			stream cr. ]! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 21:56'!
printTotals: results

	| resultsGroupedByRunner totalResults fuelTotalResultOrNil |
	resultsGroupedByRunner := results groupedBy: [ :x | x run runner ].
	
	totalResults := resultsGroupedByRunner 
		collect: [ :runnerResults | self totalResultFor: runnerResults anyOne run runner in: runnerResults ].

	fuelTotalResultOrNil := 
		totalResults
			detect: [:aTotalResult | aTotalResult runner isFuelRunner] 
			ifNone: [ nil ].

	totalResults do: [ :totalResult |
		totalResult 
			printResultsOn: stream 
			comparingToFuelResults: fuelTotalResultOrNil ]! !

!FLBenchmarksClassicPrinter methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:54'!
stream
	^ stream! !

!FLBenchmarksClassicPrinter methodsFor: 'accessing' stamp: 'MartinDias 6/2/2011 19:54'!
stream: anObject
	stream := anObject! !

!FLBenchmarksClassicPrinter methodsFor: 'printing' stamp: 'MartinDias 6/2/2011 20:27'!
totalResultFor: aRunner in: results

	| totalTimeSerialization totalTimeMaterialization |
	totalTimeSerialization := 
		(results select: [ :aResult | aResult isSerialization ]) 
			sum: [ :aResult | aResult milliseconds ].
	totalTimeMaterialization := 
		(results select: [ :aResult | aResult isMaterialization ]) 
			sum: [ :aResult | aResult milliseconds ].

	^ FLBenchmarkTotalResultForRunner 
		runner: aRunner 
		totalTimeSerialization: totalTimeSerialization 
		totalTimeMaterialization: totalTimeMaterialization.
	! !

Object subclass: #FLDictionaryBenchmarks
	instanceVariableNames: 'dictionaryClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLDictionaryBenchmarks commentStamp: 'MartinDias 5/30/2011 03:24' prior: 0!
Print it on each line:

FLDictionaryBenchmarks resultsOfHash. 
FLDictionaryBenchmarks resultWithFastHashSample. 
FLDictionaryBenchmarks resultWithSlowHashSample. !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MarianoMartinezPeck 10/24/2011 16:51'!
newWith: aDictionaryClass

	^self basicNew 
		initializeWith: aDictionaryClass;
		yourself! !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/19/2011 15:40'!
resultWithFastHashSample
	
	|benchs|
	benchs := (self newWith: Dictionary).
	^benchs runWith: benchs smallintegersAndStringsSampleDictionary. ! !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/19/2011 15:40'!
resultWithSlowHashSample
	
	|benchs|
	benchs := (self newWith: Dictionary).
	^benchs runWith: benchs stringsAndSmallintegersSampleDictionary. ! !

!FLDictionaryBenchmarks classSide methodsFor: 'instance creation' stamp: 'MartinDias 5/19/2011 15:54'!
resultsOfHash

	| aSmallIntegerCollection aStringCollection |
	aSmallIntegerCollection := (1 to: (1 << 19)) asOrderedCollection.
	aStringCollection := ((1 to: (1 << 19)) collect: [:x | x printString ]) asOrderedCollection.

	^ 'small integers: ', [ aSmallIntegerCollection do: [:x | x hash ] ] timeToRun printString, 'ms. ',
	'strings: ', [ aStringCollection do: [:x | x hash ] ] timeToRun printString, 'ms.'.
! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MartinDias 5/19/2011 15:48'!
basicCopyOf: anObject
	"Code extracted  from Object>>shallowCopy"

	|class newObject index |

	class := anObject class.
	class isVariable
		ifTrue: 
			[index := anObject basicSize.
			newObject := class basicNew: index.
			[index > 0]
				whileTrue: 
					[newObject basicAt: index put: (anObject basicAt: index).
					index := index - 1]]
		ifFalse: [newObject := class basicNew].
	index := class instSize.
	[index > 0]
		whileTrue: 
			[newObject instVarAt: index put: (anObject instVarAt: index).
			index := index - 1].
	^ newObject! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MartinDias 5/19/2011 15:56'!
copyRecreating: sourceDictionary

	| copy |
	copy := sourceDictionary class new: sourceDictionary size.
	
	sourceDictionary associationsDo: [:assoc | 
		copy at: assoc key put: assoc value ].
! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MartinDias 5/19/2011 15:56'!
copyRehashing: sourceDictionary 

	(self basicCopyOf: sourceDictionary) rehash! !

!FLDictionaryBenchmarks methodsFor: 'private' stamp: 'MarianoMartinezPeck 10/24/2011 16:52'!
initializeWith: aDictionaryClass
	super initialize.
	dictionaryClass := aDictionaryClass! !

!FLDictionaryBenchmarks methodsFor: 'testing' stamp: 'MartinDias 5/19/2011 15:28'!
runWith: sourceDictionary 

	^ #(copyRecreating: copyRehashing:)
		inject: '' 
		into: [:result :selector | 
			result, 
			selector printString, 
			' run ms = ', 
			[self perform: selector with: sourceDictionary] 
				timeToRun printString ].
! !

!FLDictionaryBenchmarks methodsFor: 'testing' stamp: 'MartinDias 5/19/2011 15:51'!
smallintegersAndStringsSampleDictionary

	| sourceDictionary|
	sourceDictionary := dictionaryClass new.
	1 to: (1 << 19) do:  [:aNumber | sourceDictionary at: aNumber put: aNumber printString ] .
	^sourceDictionary ! !

!FLDictionaryBenchmarks methodsFor: 'testing' stamp: 'MartinDias 5/19/2011 15:51'!
stringsAndSmallintegersSampleDictionary

	| sourceDictionary|
	sourceDictionary := dictionaryClass new.
	1 to: (1 << 19) do:  [:aNumber | sourceDictionary at: aNumber printString put: aNumber ] .
	^sourceDictionary ! !

Object subclass: #FLExecutionBenchmarks
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks'!

!FLExecutionBenchmarks commentStamp: 'MartinDias 9/17/2011 19:55' prior: 0!
Open Trascript and then evaluate: 

FLExecutionBenchmark benchs.!

!FLExecutionBenchmarks classSide methodsFor: 'benchmarking' stamp: 'MartinDias 9/17/2011 20:04'!
benchs

	(self class methodsInCategory: 'executing')
		do: [:m |
			Transcript 
				cr; 
				show: m printString;
				show: '->'.
			[self perform: m] bench printOn: Transcript ].
	Transcript flush.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:01'!
block1

	^[ Integer ] value.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:01'!
block2

	^[ Smalltalk globals at: #Integer ] value.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:00'!
block3

	^ ([:x | x globals ] value: Smalltalk) at: #Integer.! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 20:00'!
block4

	^ [:y :z| y at: z ] 
		value: ([:x | x globals ] value: Smalltalk)
		value: #Integer
! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 19:47'!
compiled1

	^ Integer! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 19:46'!
compiled2

	^ Smalltalk globals at: #Integer! !

!FLExecutionBenchmarks classSide methodsFor: 'executing' stamp: 'MartinDias 9/17/2011 19:48'!
perform1

	^ (Smalltalk 
		perform: #globals) 
		perform: #at: with: #Integer! !

Object subclass: #FLHookSubstitutedObject
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelBenchmarks-Samples'!

!FLHookSubstitutedObject classSide methodsFor: 'instance creation' stamp: 'MartinDias 1/7/2012 14:28'!
with: anObject

	^ self basicNew
		initializeWith: anObject;
		yourself.! !

!FLHookSubstitutedObject methodsFor: 'hooks' stamp: 'MartinDias 1/9/2012 15:23'!
fuelAccept: aVisitor
	^ aVisitor visitSubstitution: self by: contents! !

!FLHookSubstitutedObject methodsFor: 'initialize-release' stamp: 'MartinDias 1/7/2012 14:27'!
initializeWith: anObject

	self initialize.
	contents := anObject.! !
